# Copyright (c) 2002
#      The Regents of the University of California.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that: (1) source code distributions
# retain the above copyright notice and this paragraph in its entirety, (2)
# distributions including binary code include the above copyright notice and
# this paragraph in its entirety in the documentation or other materials
# provided with the distribution, and (3) all advertising materials mentioning
# features or use of this software display the following acknowledgement:
# ``This product includes software developed by the University of California,
# Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
# the University nor the names of its contributors may be used to endorse
# or promote products derived from this software without specific prior
# written permission.
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# This code was written by Umesh Shankar <ushankar@cs.berkeley.edu>
# Please contact me with questions, comments, bug fixes, etc.
#

#
# The Firewall packages manages a pool of source TCP ports
# for use in user-level TCP. It does the necessary manipulation of the kernel
# firewall to keep the kernel from seeing traffic to the pool, also generates a pcap
# filter so that traffic to those ports can be captured at user level
#

package ActiveMap::Firewall;

my $_base_port = undef;
my $_last_idx_allocated;
my @port_in_use;
my $fw;

# sets the firewall, returns the pcap filter
sub set_port_range
  {
    my ($low, $high) = @_;

    $_base_port = $low;
    $_last_idx_allocated = int(rand($high-$low));
    @port_in_use = (0) x ($high-$low+1);

    warn "# Firewall must be manually set up to filter out TCP ports $low => $high";

    return "dst host " . ActiveMap::Utils::this_ip() . " and tcp " .
      "and tcp[2:2] >= $low and tcp[2:2] <= $high";

#      my $fw_pkg = ( ($^O eq 'freebsd') && ActiveMap::Firewall::BSD or
#  		   ($^O eq 'linux') && ActiveMap::Firewall::Linux or
#  		   die "Only Linux and freebsd supported" );

#      use $fw_pkg;

#      $fw = new $fw_pkg;

#      $fw->set_tcp_firewall($low, $high);

  }

# ask for use of a source port, returns number of port allocated
sub get_tcp_source_port
  {
    die "call set_port_range before other fns in ActiveMap::Firewall"
      if ! defined $_base_port;

    if ($_last_idx_allocated < @port_in_use - 1 &&
	! $port_in_use[$_last_idx_allocated + 1])
      {
	++$_last_idx_allocated;
	$port_in_use[$_last_idx_allocated] = 1;
	return $_last_idx_allocated + $_base_port;
      }

    for (my $i = 0; $i < @port_in_use; $i++)
      {
	if (!$port_in_use[$i])
	  {
	    $_last_idx_allocated = $i;
	    $port_in_use[$i] = 1;
	    return $i + $_base_port;
	  }
      }

    die "No source ports available!";
  }

# returns a port to the list of available ones
sub free_tcp_source_port
  {
    my $port_number = shift;
    $port_in_use[$port_number - $_base_port] = 0;
  }


sub num_ports_allocated
  {
    return scalar(@port_in_use);
  }

sub get_filter_str
  {
    return "(tcp and (tcp[2:2] >= $_base_port) and " .
      "(tcp[2:2] <= " . ($_base_port + scalar(@port_in_use)) ."))";
  }

1;
