#!/usr/bin/perl

# Copyright (c) 2002
#      The Regents of the University of California.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that: (1) source code distributions
# retain the above copyright notice and this paragraph in its entirety, (2)
# distributions including binary code include the above copyright notice and
# this paragraph in its entirety in the documentation or other materials
# provided with the distribution, and (3) all advertising materials mentioning
# features or use of this software display the following acknowledgement:
# ``This product includes software developed by the University of California,
# Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
# the University nor the names of its contributors may be used to endorse
# or promote products derived from this software without specific prior
# written permission.
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# This code was written by Umesh Shankar <ushankar@cs.berkeley.edu>
# Please contact me with questions, comments, bug fixes, etc.
#

use strict;

my @classes; # [ regex, classname ]
my @hosts; # { ip=> , hostname=>, class => }

sub get_class
  {
    my $hostname = shift;
    foreach (@classes)
      {
	return $_->[1] if ($hostname =~ /$_->[0]/);
      }
    return "unknown";
  }

sub usage
  {
    print STDERR 
'perl classify.pl CLASSFILE HOSTFILE
where CLASSFILE is a file with line pairs like
(print)|(lpr)
printer
where the first line of the pair is a perl regex and the second the class name

and HOSTFILE is a file where each line has an IP as the first field
and the second field as the hostname. 

The output of classify.pl is a the original file with another field
which is the classification of the host based on matching regexes
against the name
';

    exit(-1);
  }

open CLASSFILE, $ARGV[0] or usage();
open HOSTFILE,  $ARGV[1] or usage();

while (<CLASSFILE>)
  {
    chomp;
    my $regex = $_;
    chomp($_ = <CLASSFILE>);
    my $classname = $_;
    push @classes, [ $regex, $classname ];
  }

while (<HOSTFILE>)
  {
    my ($ip, $hostname) = split;
    push @hosts, 
      { ip => $ip, hostname => $hostname, class => get_class($hostname) };
  }

printf "%s\t%40s\t%s\n", $_->{ip}, $_->{hostname},$_->{class}
  foreach (@hosts);
