#!/usr/bin/perl

# Copyright (c) 2002
#      The Regents of the University of California.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that: (1) source code distributions
# retain the above copyright notice and this paragraph in its entirety, (2)
# distributions including binary code include the above copyright notice and
# this paragraph in its entirety in the documentation or other materials
# provided with the distribution, and (3) all advertising materials mentioning
# features or use of this software display the following acknowledgement:
# ``This product includes software developed by the University of California,
# Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
# the University nor the names of its contributors may be used to endorse
# or promote products derived from this software without specific prior
# written permission.
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# This code was written by Umesh Shankar <ushankar@cs.berkeley.edu>
# Please contact me with questions, comments, bug fixes, etc.
#

my $USAGE = 
'Usage uname.pl blah.uname 0|1|2 parsefile\n
uname.pl combines results of parse.pl with uname info
There are three uname types, based on the format of each line in the 
uname file:
   ip hostname shortname ostype shortname osver ... (uname_type = 0)
   ip [name:]ostype name osver ... (uname_type = 1) 
 (tab-delimited) ip hostname ostype opt_osver (uname_type = 2) 
If parsefile is "-" or not specified, then stdin is used.
Output goes to stdout.
';

use strict;
use Text::ParseWords;

my ($uname_file, $uname_type, $parsetable) = @ARGV;

if (! defined $uname_file)
  {
    print STDERR $USAGE;
    exit(0);
  }

my $parse;
if ($parsetable eq '-' || !defined $parsetable)
  {
    *PARSE = *STDIN;
  }
else { open PARSE, $parsetable or die "could not open parse file\n" }


my %osmap;

#  open UNAME, $uname_file or die "could not open uname file\n";
#  my $sample = <UNAME>;
#  my ($firstcol) = $sample =~ /^(\S+)/;
#  if ($firstcol =~ /[0-9.]+/)
#    {
#      $uname_type = 0;
#    }
#  else
#    {
#      $uname_type = 1;
#    }
#  close UNAME;

open UNAME, $uname_file or die "could not open uname file\n";

while (<UNAME>)
  {
    my ($ip, $hostname, $shortname1, $ostype, $shortname2, $osver);
    if ( $uname_type == 0 )
      {
	($ip, $hostname, $shortname1, $ostype, $shortname2, $osver) = 
	  split /\s+/;
      }
    elsif ( $uname_type == 1 )
      {
	($ip, $ostype, $hostname, $osver) = split /\s+/ if $uname_type == 1;
	$ostype =~ s/^.*://;
      }
    elsif ( $uname_type == 2 )
      {
	($ip, $hostname, $ostype, $osver) = parse_line("\t", 0, $_);
	foreach my $field ( $ip, $hostname, $ostype, $osver )
	  {
	    $field =~ s/\s+$//;
	  }
      }

    $osmap{$ip} = "$ostype $osver";
  }

while (<PARSE>)
{
  last if /\S/;
}
s/^Unknown OS//; print "OS\t", $_; # print header row

while (<PARSE>)
  {
    chomp;
#    print $_, "\t";
    my ($ip) = ($_ =~ /^([0-9.]+)/);
    print (($osmap{$ip} || "Unknown OS"), "\t", $_, "\n");
  }

